from flask_restful import fields
from flask_restful_swagger import swagger
from sqlalchemy.orm.attributes import InstrumentedAttribute
from sqlalchemy.sql import sqltypes


class Meta(type):
    MODEL_NAME = '__model_class__'
    IGNORE_FIELDS = '__ignore_fields__'
    EXTRA_FIELDS = '__extra_fields__'

    def __new__(cls, name, bases, attributes):
        model = attributes.get(Meta.MODEL_NAME)
        resource_fields = {}
        if model:
            ignore_fields = attributes.get(Meta.IGNORE_FIELDS, [])
            raw_fields = [x for x in model.__dict__ if x not in ignore_fields and not x.startswith('_')]
            for field in raw_fields:
                value = getattr(model, field)
                if isinstance(value, InstrumentedAttribute):
                    field_type = value.comparator.type
                    if isinstance(field_type, (sqltypes.String, sqltypes.Text)):
                        resource_fields[field] = fields.String
                    elif isinstance(field_type, sqltypes.Integer):
                        resource_fields[field] = fields.Integer
                    elif isinstance(field_type, sqltypes.Boolean):
                        resource_fields[field] = fields.Boolean
                    elif isinstance(field_type, sqltypes.DateTime):
                        resource_fields[field] = fields.DateTime(dt_format='iso8601')
                    elif isinstance(field_type, sqltypes.DECIMAL):
                        resource_fields[field] = fields.MyDecimal
                    else:
                        resource_fields[field] = fields.String

        # add extra fields
        extra_fields = attributes.get(Meta.EXTRA_FIELDS, {})
        if isinstance(extra_fields, dict):
            resource_fields.update(extra_fields)
        attributes['resource_fields'] = resource_fields
        meta = type.__new__(cls, name, bases, attributes)
        swagger.add_model(meta)
        return meta



from decimal import Decimal

from dateutil.parser import parse

LITERAL = {'false': False, 'true': True}


def compatible_bool(value):
    """it's used to reqparser.RequestParser

    Example:
        >>> from flask_restful import reqparse
        >>> parser = reqparse.RequestParser()
        >>> parser.add_argument('marriaged', type=compatible_bool, help='accept false and true string')
    :param value:
    :return:
    """
    if isinstance(value, basestring):
        json_value = value.lower()
        if json_value in LITERAL:
            return LITERAL.get(json_value)
    return bool(value)


def compatible_datetime(value, name):
    return parse(value)


def compatible_decimal(value):
    try:
        return Decimal(value)
    except Exception:
        return None



from .bunch import Bunch
from .pay import make_company_pay
from .pay import make_h5_pay_sign
from .pay import make_prepay_id
from .pay import make_response_content
from .pay import parse_company_pay_result
from .pay import parse_prepay_result
from .pay import valid_message
from .rand import new_validate_code
from .rand import nonce_str
from .template_message import TemplateMessage
from .util import get_raw_path
from .util import purge_sum_result
from .util import timestamp
from .validation import is_cell_phone



# -:- coding:utf8 -:-

from flask_login import login_required
from flask_restful import Resource

from flask_kits.decorators import manager_required
from flask_kits.decorators import wx_required


class WxBusinessResource(Resource):
    method_decorators = [wx_required, login_required]


class BusinessResource(Resource):
    method_decorators = [login_required, manager_required]



def filter_params():
    return [
        {
            "name": "page",
            "description": "page index used to pagination",
            "required": False,
            "dataType": "int",
            "paramType": "query"
        }, {
            "name": "per_page",
            "description": "page size per page",
            "required": False,
            "dataType": "int",
            "paramType": "query"
        }
    ]



from flask_restful import fields
from flask_restful import marshal_with
from flask_restful_swagger import swagger
from six import add_metaclass

from flask_kits.restful import Paginate
from flask_kits.restful import filter_params
from .swagger import post_parameter


class LocalDateTime(fields.DateTime):
    def format(self, value):
        result = super(LocalDateTime, self).format(value)
        return result.replace('T', ' ')


MAPPING = {
    'integer': fields.Integer,
    'boolean': fields.Boolean,
    'date': LocalDateTime(dt_format='iso8601'),
    'datetime': LocalDateTime(dt_format='iso8601')
}


class SerializerMetaclass(type):
    def __new__(cls, name, bases, attributes):
        if name == 'Serializer':
            return type.__new__(cls, name, bases, attributes)

        model = attributes.pop('__model__', None)  # type: User
        class_dict = attributes.copy()
        class_dict['resource_fields'] = resource_fields = {}
        if model:
            for column in model.__table__.columns:
                field_type_name = column.type.__visit_name__
                field_type = MAPPING.get(field_type_name)
                if not field_type:
                    field_type = fields.String
                resource_fields[column.name] = field_type
        s = type.__new__(cls, name, bases, class_dict)
        swagger.add_model(s)
        return s


@add_metaclass(SerializerMetaclass)
class Serializer(object):
    @classmethod
    def operation(cls, f, paginate=True):
        attr = {
            'notes': f.__doc__,
            'nickname': f.__name__,
            'responseClass': cls,
            'parameters': filter_params() if paginate else []
        }

        return attr

    @classmethod
    def parameter(cls, name, description=None, data_type='str', param_type='query', required=False):
        def decorator(func):
            attr = func.__dict__['__swagger_attr']
            params = attr.get('parameters', [])
            # TODO(benjamin): check data_type type
            if type(data_type).__name__ == 'type' and issubclass(data_type, cls):
                params.append(post_parameter(data_type))
            else:
                params.append({
                    "name": name,
                    "description": description or name,
                    "required": required,
                    "dataType": str(data_type),
                    "paramType": param_type
                })
            attr['parameters'] = params
            return func

        return decorator

    @classmethod
    def list(cls, item_builder=None):
        def decorator(func):
            wrapper = Paginate(cls.resource_fields, item_builder=item_builder)
            wrapper = wrapper(func)
            wrapper.__dict__['__swagger_attr'] = cls.operation(func)
            return wrapper

        return decorator

    @classmethod
    def single(cls, func):
        wrapper = marshal_with(cls.resource_fields)(func)
        wrapper.__dict__['__swagger_attr'] = cls.operation(func, paginate=False)
        return wrapper

    def __str__(self):
        return self.__class__.__name__

    def __repr__(self):
        return self.__class__.__name__



import hashlib
import os.path

from flask import current_app
from werkzeug.routing import Rule


class KitRule(Rule):
    KNAPSACK = {}

    def build(self, values, append_unknown=True):
        if self.endpoint == 'static' and current_app.has_static_folder and 'filename' in values:
            filename = values.get('filename')
            if filename in KitRule.KNAPSACK:
                values['v'] = KitRule.KNAPSACK.get(filename)
            else:
                static_file_path = os.path.join(current_app.static_folder, filename)
                static_file_path = os.path.normpath(static_file_path)
                if os.path.exists(static_file_path):
                    with open(static_file_path, 'rb') as f:
                        version = hashlib.sha1(f.read()).hexdigest()[:8]
                        KitRule.KNAPSACK[filename] = version
                        values['v'] = version

        return super(KitRule, self).build(values, append_unknown=append_unknown)



import random
import string


def nonce_str():
    terms = string.letters + string.digits
    return ''.join([random.choice(terms) for _ in xrange(16)])


def new_validate_code():
    terms = string.digits
    return ''.join([random.choice(terms) for _ in xrange(6)])



# -:- coding:utf8 -:-
"""

"""

from flask_kits.routing import KitRule

__author__ = 'benjamin.c.yan'
__version__ = '0.0.4'

missing = object()


class Kits(object):
    PREFIX = 'KITS'

    def __init__(self, app=None, **kwargs):
        # from flask import Flask
        # Flask.url_rule_class = KitRule
        self._app = None
        self.app_name = 'recipe'
        self._options = {}
        if app:
            self.init_app(app, **kwargs)

    def init_app(self, app, **kwargs):
        self._app = app
        self._options.update(kwargs)
        self.app_name = self.get_parameter('app_name')

        if not hasattr(app, 'extensions'):
            app.extensions = {}

            # jsapi.init_extension(self, app)
            # wxapi.init_extension(self, app)
            # sms.init_extension(self, app)
            # tm.init_extension(self, app)

    def get_parameter(self, name, default=None):
        option = self._options.get(name, missing)
        app = self._app
        if option is missing:
            parameter_name = '{0}_{1}'.format(Kits.PREFIX, name.upper())
            option = app.config.get(parameter_name, default)
        return option



# -*- coding:utf-8 -*-
import hashlib

__author__ = 'benjamin.c.yan'


def sign(params, api_key):
    """
    signature api
    :param params: [(`class`:`str`, `class`:`object`)]
    :return: `class`:`str`
    """
    params = [(key, value) for key, value in params if value]
    params = sorted(params, key=lambda param: param[0])
    temp = u'&'.join([u'{0}={1}'.format(key, value) for key, value in params])
    temp = u'{temp}&key={key}'.format(temp=temp, key=api_key)
    return hashlib.md5(temp.encode('utf8')).hexdigest().upper()


if __name__ == '__main__':
    key1 = sign([('body', u'中文')], '192006250b4c09247ec02edce69f6a2d')
    # assert key1 == '9A0A8659F005D6984697E2CA0A9CF3B7'
    print key1
    key1 = sign([('appid', 'wxd930ea5d5a258f4f'), ('mch_id', '10000100'), ('device_info', '1000'), ('body', 'test'),
                 ('nonce_str', 'ibuaiVcKdpRxkhJA')], '192006250b4c09247ec02edce69f6a2d')
    assert key1 == '9A0A8659F005D6984697E2CA0A9CF3B7'
    print key1

    key1 = sign([('appid', 1)], '192006250b4c09247ec02edce69f6a2d')
    print key1



# -*- coding: utf-8 -*-

ENCODINGS = ['utf8', 'gbk']


def decode_statement(statement, encodings):
    if isinstance(statement, unicode):
        return statement
    for encoding in encodings:
        try:
            return statement.decode(encoding)
        except UnicodeDecodeError:
            pass


def get_initial_letters(statement):
    statement = decode_statement(statement, ENCODINGS)
    if statement is None:
        return ''
    return ''.join(get_initial_letter(word) for word in statement)


def get_initial_letter(character):
    character = character.encode('gbk')
    try:
        ord(character)
        return character.lower()
    except Exception:
        # ignore exception
        asc = ord(character[0]) * 256 + ord(character[1]) - 65536
        if -20319 <= asc <= -20284:
            return 'a'
        if -20283 <= asc <= -19776:
            return 'b'
        if -19775 <= asc <= -19219:
            return 'c'
        if -19218 <= asc <= -18711:
            return 'd'
        if -18710 <= asc <= -18527:
            return 'e'
        if -18526 <= asc <= -18240:
            return 'f'
        if -18239 <= asc <= -17923:
            return 'g'
        if -17922 <= asc <= -17418:
            return 'h'
        if -17417 <= asc <= -16475:
            return 'j'
        if -16474 <= asc <= -16213:
            return 'k'
        if -16212 <= asc <= -15641:
            return 'l'
        if -15640 <= asc <= -15166:
            return 'm'
        if -15165 <= asc <= -14923:
            return 'n'
        if -14922 <= asc <= -14915:
            return 'o'
        if -14914 <= asc <= -14631:
            return 'p'
        if -14630 <= asc <= -14150:
            return 'q'
        if -14149 <= asc <= -14091:
            return 'r'
        if -14090 <= asc <= -13119:
            return 's'
        if -13118 <= asc <= -12839:
            return 't'
        if -12838 <= asc <= -12557:
            return 'w'
        if -12556 <= asc <= -11848:
            return 'x'
        if -11847 <= asc <= -11056:
            return 'y'
        if -11055 <= asc <= -10247:
            return 'z'
        return ''


if __name__ == "__main__":
    x = u'迦舒布鲁姆Ⅰ峰'
    print(get_initial_letters(x))



from functools import partial

from bunch import Bunch

PREFIX_WX = "wx"
PREFIX_MANAGER = "manager"


def wrap_user_id(prefix, user_id):
    return "{prefix}_{id}".format(prefix=prefix, id=user_id)


wrap_user = partial(wrap_user_id, PREFIX_MANAGER)
wrap_wx = partial(wrap_user_id, PREFIX_WX)


def unwrap(identity):
    """
    :param str identity:
    :rtype Bunch
    """
    if not identity:
        return None

    prefix, delimiter, user_id = identity.partition('_')
    if not delimiter:
        return None

    return Bunch(id=int(user_id), is_wx_user=prefix == PREFIX_WX)



from functools import partial

from bunch import Bunch
from flask import request
from flask_login import current_user
from flask_menu import MenuEntryMixin
from flask_menu import current_menu
from flask_menu import register_menu


def visible_when(roles):
    """

    :param list[str] roles: 
    :return: 
    """
    if not roles:
        return True

    actual_roles = set(roles)
    expect_roles = set()
    if hasattr(current_user, 'role'):
        expect_roles.add(current_user.role.name)

    if hasattr(current_user, 'roles'):
        for role in current_user.roles:
            expect_roles.add(role.name)
    return actual_roles & expect_roles


def active_when(menu_item):
    """

    :param MenuEntryMixin menu_item: 
    :return: 
    """
    if request.endpoint == menu_item._endpoint:
        return True

    return False


def config_menu(app, items):
    """
    items contains menu item, like below
    {'name': 'profile', 'text': 'Home', 'roles': ['admin'], 'order': 1}

    :param flask.Flask app:
    :param list[Bunch] items:

    """
    if not items:
        return

    @app.before_first_request
    def before_first_request():
        for item in items:
            name = item.pop('name')
            menu_item = current_menu.submenu(name)  # type: MenuEntryMixin
            item.endpoint = None
            item.visible_when = partial(visible_when, menu_item)
            # kwargs['active_when'] = active_when
            menu_item.register(**item)


def register_menu_ex(app, path, text, **kwargs):
    """
    :param app:
    :param path:
    :param text:
    :param kwargs:

    """
    new_visible_when = partial(visible_when, kwargs.get('roles'))
    kwargs['visible_when'] = new_visible_when
    return register_menu(app, path, text, **kwargs)

register_menu = partial(register_menu, visible_when=visible_when)



# -*- coding:utf-8 -*-
import logging
from datetime import datetime

import requests


class TemplateMessage(object):
    logger = logging.getLogger('drp.muggle')
    _session = requests.session()

    @staticmethod
    def make_timestamp(today=None):
        if today is None:
            today = datetime.now()
        return u"{0}年{1}月{2}日 {3:02}:{4:02}:{5:02}".format(today.year, today.month, today.day, today.hour,
                                                           today.minute, today.second)

    @staticmethod
    def build_retrieve_template_id_url(access_token):
        return 'https://api.weixin.qq.com/cgi-bin/template/api_add_template?access_token={0}'.format(access_token)

    @staticmethod
    def build_send_message_url(access_token):
        return 'https://api.weixin.qq.com/cgi-bin/message/template/send?access_token={0}'.format(access_token)

    @staticmethod
    def send_message(message_id, open_id, **kwargs):
        from flask_kits import JsApi as  access
        access_token = access.token

        try:
            url = TemplateMessage.build_send_message_url(access_token)
            pay_load = {
                "touser": open_id,
                "template_id": message_id,
                "url": "{0}/drp/h5/ordinary_member".format(settings.APP_DOMAIN),
                'data': {key: dict(value=value) for key, value in kwargs.iteritems()}}
            response = TemplateMessage._session.post(url, json=pay_load, verify=False)
            result = response.json()
            if result['errcode']:
                TemplateMessage.logger.error(u'send template message failure, error code: %s, error message: %s',
                                             result['errcode'], result['errmsg'])
                return False
            return True
        except Exception, e:
            TemplateMessage.logger.exception(e)
            return False

    @staticmethod
    def send_pay_message(open_id, follower_nick_name, item_name, price):

        kwargs = dict(first=u"你的下级已购买一件商品",
                      keyword1=follower_nick_name,
                      keyword2=item_name,
                      keyword3=u"{0} 元".format(price),
                      keyword4=TemplateMessage.make_timestamp(),
                      remark=u"购买成功后你将获得返利")
        return TemplateMessage.send_message(settings.TPL_PAY, open_id, **kwargs)

    @staticmethod
    def send_new_follow_message(open_id, follower_nick_name):
        kwargs = dict(first=u"你新增了一个下级会员",
                      keyword1=follower_nick_name,
                      keyword2=TemplateMessage.make_timestamp(),
                      remark=u"")
        return TemplateMessage.send_message(settings.TPL_NEW_FOLLOW, open_id, **kwargs)

    @staticmethod
    def send_bonus_message(open_id, item_name, total_bonus, voucher, order_cost, available_date):
        kwargs = dict(first=u"你有下级会员购买成功，给你的返利已到账",
                      keyword1=item_name,
                      keyword2=str(total_bonus),
                      keyword3=str(order_cost),
                      keyword4=TemplateMessage.make_timestamp(available_date),
                      remark=u"其中有{0}元是代金券".format(voucher))
        return TemplateMessage.send_message(settings.TPL_BONUS, open_id, **kwargs)


if __name__ == '__main__':
    TemplateMessage.send_pay_message('o7unIwPw8iiTSvQpQe6QcMRtJapA', follower_nick_name="Neo",
                                     item_name="Demo product", price=12.19)



from functools import wraps

from flask import abort
from flask import has_request_context
from flask import request
from flask_restful import fields
from flask_restful import marshal
from flask_restful import marshal_with
from flask_restful.utils import unpack
from flask_sqlalchemy import BaseQuery
from flask_sqlalchemy import Pagination
from six.moves import zip
from six.moves.urllib_parse import urlparse
from six.moves.urllib_parse import urlunsplit


def paginate(query, page=None, per_page=None, error_out=True):
    """Clone from flask_sqlachemy.Pagination
    Returns `per_page` items from page `page`.  By default it will
    abort with 404 if no items were found and the page was larger than
    1.  This behavor can be disabled by setting `error_out` to `False`.

    If page or per_page are None, they will be retrieved from the
    request query.  If the values are not ints and ``error_out`` is
    true, it will abort with 404.  If there is no request or they
    aren't in the query, they default to page 1 and 20
    respectively.

    Returns an :class:`Pagination` object.
    """

    if has_request_context():
        if page is None:
            try:
                page = int(request.args.get('page', 1))
            except (TypeError, ValueError):
                if error_out:
                    abort(404)

                page = 1

        if per_page is None:
            try:
                per_page = int(request.args.get('per_page', 20))
            except (TypeError, ValueError):
                if error_out:
                    abort(404)

                per_page = 20
    else:
        if page is None:
            page = 1

        if per_page is None:
            per_page = 20

    if error_out and page < 1:
        abort(404)

    items = query.limit(per_page).offset((page - 1) * per_page).all()

    if not items and page != 1 and error_out:
        abort(404)

    # No need to count if we're on the first page and there are fewer
    # items than we expected.
    if page == 1 and len(items) < per_page:
        total = len(items)
    else:
        total = query.order_by(None).count()

    return Pagination(query, page, per_page, total, items)


class Paginate(marshal_with):
    """used to pagination

    Example:
        @Paginate({'id'': fields.Integer})
        def get():
            pass

    """

    def __init__(self, serializer_fields, envelope=None, item_builder=None):
        """decorator initialize for Paginate

        :param serializer_fields: a dict of whose keys will make up the final
                       serialized response output
        :param envelope: optional key that will be used to envelop the serialized
                         response
        :param item_builder: optional key that will be used to rebuild item for group_by
        :return:
        """
        super(Paginate, self).__init__(self.rebuild_fields(serializer_fields), envelope=envelope)
        self.item_builder = item_builder

    def __call__(self, func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            resp = func(*args, **kwargs)
            if isinstance(resp, tuple):
                query, code, headers = unpack(resp)
                data = self.make_pagination(query)
                return marshal(data, self.fields, self.envelope), code, headers
            else:
                data = self.make_pagination(resp)
                return marshal(data, self.fields, self.envelope)

        return wrapper

    @staticmethod
    def rebuild_fields(serializer_fields):
        return {
            'count': fields.Integer,
            'previous': fields.String,
            'next': fields.String,
            'results': fields.List(fields.Nested(serializer_fields))
        }

    @staticmethod
    def make_query(url, page_number, per_page):
        segments = urlparse(url)
        querystring = 'page={page}&per_page={per_page}'.format(page=page_number, per_page=per_page)
        return urlunsplit((segments.scheme, segments.netloc, segments.path, querystring, segments.fragment))

    def make_pagination(self, query):
        if isinstance(query, MultiQueryResult):
            pagination_list = [paginate(q) for q in query.results]
            items_list = [p.items for p in pagination_list]
            pagination = pagination_list[0]
            pagination.items = [self.item_builder(record) for record in zip(*items_list)]

        else:
            if isinstance(query, BaseQuery):
                pagination = query.paginate()
            else:
                pagination = paginate(query)

            if self.item_builder:
                builder = self.item_builder
                pagination.items = [builder(item) for item in pagination.items]

        # segments = urlparse(request.url)
        # url = furl(request.url)
        # url.args['per_page'] = pagination.per_page
        previous_url = None
        if pagination.has_prev:
            # url.args['page'] = pagination.prev_num
            previous_url = self.make_query(request.url, pagination.prev_num, pagination.per_page)

        next_url = None
        if pagination.has_next:
            # url.args['page'] = pagination.next_num
            next_url = self.make_query(request.url, pagination.next_num, pagination.per_page)

        return {
            'count': pagination.total,
            'previous': previous_url,
            'next': next_url,
            'results': pagination.items
        }


class MultiQueryResult(object):
    def __init__(self, *args):
        self.results = args



from .filter import filter_params
from .meta import Meta
from .pagination import MultiQueryResult
from .pagination import Paginate
from .parser import compatible_bool
from .parser import compatible_datetime
from .parser import compatible_decimal
from .resource import BusinessResource
from .resource import WxBusinessResource
from .serializer import LocalDateTime
from .serializer import Serializer
from .swagger import post_parameter



import logging
from xml.etree import ElementTree as ET

from .rand import nonce_str
from .signature import sign
from .util import timestamp

SUCCESS = 'success'
FAIL = 'fail'

logger = logging.getLogger('drp.muggle')


def make_h5_pay_sign(app_id, pre_pay_id, api_key):
    params = [('appId', app_id), ('timeStamp', timestamp()), ('nonceStr', nonce_str()),
              ('package', 'prepay_id={0}'.format(pre_pay_id)),
              ('signType', 'MD5')]
    h5_sign = sign(params, api_key)
    params.append(('sign', h5_sign))
    params.append(('pre_pay_id', pre_pay_id))
    return dict(params)


def make_prepay_id(entity, api_key):
    """
    appid, mch_id, body

    :param entity: `class`:`Bunch`
    :param api_key: `class`:`str`
    :return:  `class`:`str`
    """
    nonce = nonce_str()
    params = [('appid', entity.appid),
              ('mch_id', entity.mch_id),
              ('device_info', 'WEB'),
              ('nonce_str', nonce),
              ('body', entity.body),
              ('detail', entity.detail),
              ('out_trade_no', entity.out_trade_no),
              ('total_fee', entity.total_fee),
              ('spbill_create_ip', entity.spbill_create_ip),
              ('time_start', entity.time_start),
              ('time_expire', entity.time_expire),
              ('notify_url', entity.notify_url),
              ('trade_type', 'JSAPI'),
              ('openid', entity.openid)]
    pay_sign = sign(params, api_key)
    params.append(('sign', pay_sign))

    root = ET.Element('xml')

    for key, value in params:
        node = ET.SubElement(root, key)
        if isinstance(value, unicode):
            node.text = value.encode('utf8')
        else:
            node.text = str(value)

    return ET.tostring(root)


def make_company_pay(entity, api_key):
    nonce = nonce_str()
    params = [('mch_appid', entity.get('appid')),
              ('mchid', entity.get('mch_id')),
              ('device_info', 'WEB'),
              ('nonce_str', nonce),
              ('partner_trade_no', entity.get('partner_trade_no')),
              ('openid', entity.get('openid')),
              ('check_name', 'NO_CHECK'),
              ('amount', entity.get('amount')),
              ('desc', 'Cash out'),
              ('spbill_create_ip', entity.get('ip'))]
    pay_sign = sign(params, api_key)
    params.append(('sign', pay_sign))

    root = ET.Element('xml')

    for key, value in params:
        node = ET.SubElement(root, key)
        if isinstance(value, unicode):
            node.text = value.encode('utf8')
        else:
            node.text = str(value)

    return ET.tostring(root)


def parse_prepay_result(content):
    """
    :param content: `class`:`str`
    :return:  (`class`:`bool`, `class`:`str`)
    """
    try:
        root = ET.XML(content)
        params = dict([(e.tag, e.text) for e in root])
        if params['return_code'].lower() == FAIL:
            return False, params['return_msg']
        if params['result_code'].lower() == FAIL:
            return False, params['err_code']
        return True, params['prepay_id']
    except ET.ParseError as e:
        return False, e.message


def parse_company_pay_result(content):
    """
    :param content: `class`:`str`
    :return:  (`class`:`bool`, `class`:`str`)
    """
    try:
        root = ET.XML(content)
        params = dict([(e.tag, e.text) for e in root])
        if params['return_code'].lower() == FAIL:
            return False, params
        if params['result_code'].lower() == FAIL:
            return False, params
        return True, params
    except ET.ParseError as e:
        return False, e.message


def make_response_content(return_code='SUCCESS', return_message="OK"):
    root = ET.Element('xml')
    code = ET.SubElement(root, 'return_code')
    code.text = return_code
    message = ET.SubElement(root, 'return_msg')
    message.text = return_message
    return ET.tostring(root)


def valid_message(params, api_key):
    old_sign = params.pop('sign')
    new_sign = sign(params.items(), api_key)
    return old_sign == new_sign


if __name__ == '__main__':
    from bunch import Bunch

    entity = Bunch(appid='001', mch_id='002', body='body', detail='detail', out_trade_no='out_trade_no',
                   total_fee=10000,
                   spbill_create_ip='127.0.0.1', time_start='20091225091010', time_expire='20091225091010',
                   notify_url='http://www.weixin.qq.com/wxpay/pay.php', openid='1111111111')
    print make_prepay_id(entity, 'api_key')
    xml = """<xml>
       <return_code><![CDATA[SUCCESS]]></return_code>
       <return_msg><![CDATA[OK]]></return_msg>
       <appid><![CDATA[wx2421b1c4370ec43b]]></appid>
       <mch_id><![CDATA[10000100]]></mch_id>
       <nonce_str><![CDATA[IITRi8Iabbblz1Jc]]></nonce_str>
       <sign><![CDATA[7921E432F65EB8ED0CE9755F0E86D72F]]></sign>
       <result_code><![CDATA[SUCCESS]]></result_code>
       <prepay_id><![CDATA[wx201411101639507cbf6ffd8b0779950874]]></prepay_id>
       <trade_type><![CDATA[JSAPI]]></trade_type>
    </xml>"""
    print parse_prepay_result(xml)



from __future__ import print_function

import io
import os.path
import re
from distutils.text_file import TextFile

from setuptools import find_packages, setup

home = os.path.abspath(os.path.dirname(__file__))
missing = object()


def read_description(*files, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = [io.open(name, encoding=encoding).read() for name in files]
    return sep.join(buf)


def read_dependencies(requirements=missing):
    if requirements is None:
        return []
    if requirements is missing:
        requirements = 'requirements.txt'
    if not os.path.isfile(requirements):
        return []
    text = TextFile(requirements, lstrip_ws=True)
    try:
        return text.readlines()
    finally:
        text.close()


def read_version(module_name):
    with open(os.path.join(module_name, '__init__.py'), 'r') as fd:
        result = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                           fd.read(), re.MULTILINE)
        return result.group(1) if result else '0.0.1'


setup(
    name='flask_kits',
    version=read_version('flask_kits'),
    license='The MIT License',
    description='demo',
    author='recipe',
    author_email='recipe@newegg.com',
    install_requires=read_dependencies(),
    include_package_data=True,
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 3 - Alpha',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration',
    ]
)



def post_parameter(model_class):
    return {
        'name': 'mountain',
        'description': model_class.__doc__,
        'required': True,
        'dataType': model_class.__name__,
        'paramType': 'body'
    }



import time
from decimal import Decimal

__all__ = ['purge_sum_result', 'timestamp', 'get_raw_path']


def purge_sum_result(result, field_name='amount', default=None):
    key = '{0}__sum'.format(field_name)
    if default is None:
        default = Decimal('0')

    return result.get(key) or default


def timestamp():
    return str(int(time.time()))


def get_raw_path(request):
    path = request.path
    if 'QUERY_STRING' in request.META and request.META.get('QUERY_STRING'):
        path += '?' + request.META.get('QUERY_STRING')
    return path



from functools import wraps

from flask import abort
from flask import redirect
from flask import request
from flask import url_for
from flask_login import current_user
from furl import furl


def permission_required(permission):
    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            if not current_user.has_permission(permission):
                abort(401)
            return f(*args, **kwargs)

        return decorated_function

    return decorator


def admin_required(f):
    return permission_required()(f)


def common_required(validate_permission):
    """
    common required decorator, used to simple permission validate
    :param validate_permission: `class`:`func`
    :return:
    """

    def decorator(func):
        @wraps(func)
        def decorated_function(*args, **kwargs):
            if not validate_permission(current_user):
                abort(401)
            return func(*args, **kwargs)

        return decorated_function

    return decorator


def wx_required(func):
    """
    require wx login
    :param func:
    :return:
    """
    return common_required(lambda user: user.is_wx_user)(func)


def manager_required(func):
    """
    require portal login
    :param func:
    :return:
    """
    return common_required(lambda user: user.is_manager_user)(func)


def h5_required(func):
    @wraps(func)
    def decorated_function(*args, **kwargs):
        if not current_user.is_wx_user:
            location = url_for('h5.router')
            url = furl(location)
            end_point = request.endpoint
            view = end_point.split('.')[-1]
            redirect_url = furl(request.full_path)
            redirect_url.path = view
            url.query.params['redirect_uri'] = redirect_url.url
            return redirect(url.url)
        return func(*args, **kwargs)

    return decorated_function


def portal_required(func):
    @wraps(func)
    def decorated_function(*args, **kwargs):
        if not current_user.is_manager_user:
            location = url_for('auth.login')
            return redirect(location)
        return func(*args, **kwargs)

    return decorated_function
